// find the first macroblock (in left-to-right, top-to-bottom order)
// whose mean colour differs significantly from the one to the left
// (or top for first row)

// read PNG from file given on command line, or stdin if not given

#include <cmath>
#include <iostream>
#include <ostream>
#include <vector>

#include <Imlib2.h>

double calc_col_var(DATA32 d) {
	double col_mean = 0.0;
	for (int i = 0; i < 3; ++i)
		col_mean += (d >> (i*8)) & 0xff;
	col_mean /= 3.0;

	double col_var = 0.0;
	for (int i = 0; i < 3; ++i) {
		double diff = ((d >> (i*8)) & 0xff) - col_mean;
		col_var += diff * diff;
	}
	col_var /= 3.0;
	return col_var;
}

struct Colour {
	double r, g, b;

	Colour() : r(0.0), g(0.0), b(0.0) {}
	Colour(DATA32 col)
		: r((col >> 16) & 0xff)
		, g((col >> 8) & 0xff)
		, b(col & 0xff)
	{}

	Colour & operator+=(Colour const & rhs) {
		r += rhs.r;
		g += rhs.g;
		b += rhs.b;
		return *this;
	}

	Colour & operator/=(double val) {
		r /= val;
		g /= val;
		b /= val;
		return *this;
	}

	double diff(Colour const & rhs) {
		double rd = (rhs.r - r) / 255.0;
		double gd = (rhs.g - g) / 255.0;
		double bd = (rhs.b - b) / 255.0;
		return (rd * rd + gd * gd + bd * bd) / 3.0;
	}
};


// <name> file_to_read.png
int main(int argc, char *argv[]) {
	std::string filename("/dev/stdin");
	if (argc > 1)
		filename = argv[1];

	// load image
	Imlib_Image image = imlib_load_image(filename.c_str());
	imlib_context_set_image(image);
	int width = imlib_image_get_width();
	int height = imlib_image_get_height();
	DATA32 * data = imlib_image_get_data_for_reading_only();

	const double threshold = 0.002;

	std::vector<Colour> mean_colours;
	int row = 0, col = 0;
	double last_diff;
	while ((row < 30) && (col < 44)) {
		// compute mean colour
		Colour mean_colour;
		for (int y = row * 16; y < row * 16 + 16; ++y)
			for (int x = col * 16; x < col * 16 + 16; ++x)
				mean_colour += Colour(data[y * width + x]);
		mean_colour /= 256.0;

		mean_colours.push_back(mean_colour);

		if ((col == 0) && (row > 0)) {
			// compare to top neighbour
			last_diff = mean_colour.diff(mean_colours[(row - 1) * 44 + col]);
//			std::cout << col << "," << row << ": " << last_diff << "\n";
			if (last_diff > threshold)
				break;
		}
		else if (row > 0) {
			// compare to left neighbour
			last_diff = mean_colour.diff(mean_colours[row * 44 + col - 1]);
//			std::cout << col << "," << row << ": " << last_diff << "\n";
			if (last_diff > threshold)
				break;
		}

		// next
		++col;
		if (col == 44) {
			col = 0;
			++row;
		}
	}
	// Bash can't deal with floating point numbers, so make it a big integer instead
//	int score = static_cast<int>(round((row * 44 + col + 1.0 - last_diff) * 10000.0));
	int score = row * 44 + col;

	std::cout << score;
	std::cout << "\n";

	imlib_context_set_image(image);
	imlib_free_image();

	return 0;
}



